# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Steffen Kockel s.kockel@groovecubes.de
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

# This is your preferences dialog.
#
# Define your preferences dictionary in the __init__.main() function.
# The widget names in the PreferencesTestProjectDialog.ui
# file need to correspond to the keys in the preferences dictionary.
#
# Each preference also need to be defined in the 'widget_methods' map below
# to show up in the dialog itself.  Provide three bits of information:
#  1) The first entry is the method on the widget that grabs a value from the
#     widget.
#  2) The second entry is the method on the widget that sets the widgets value
#      from a stored preference.
#  3) The third entry is a signal the widget will send when the contents have
#     been changed by the user. The preferences dictionary is always up to
# date and will signal the rest of the application about these changes.
# The values will be saved to desktopcouch when the application closes.
#
# TODO: replace widget_methods with your own values


widget_methods = {
    'serial_port': ['get_text', 'set_text', 'changed'],
    'baudrate': ['get_text', 'set_text', 'changed'],
    'samplerate': ['get_text', 'set_text', 'changed'],
    'canvas_color': ['get_text', 'set_text', 'changed'],
    'grid_color': ['get_text', 'set_text', 'changed'],
}

import gettext
from gettext import gettext as _
gettext.textdomain('arduino-scope')


from serial import Serial
import time
import logging
logger = logging.getLogger('arduino_scope')

from arduino_scope_lib.PreferencesDialog import PreferencesDialog
from arduino_scope_lib.preferences import preferences
from arduino_scope.helper import testme

class PreferencesArduinoScopeDialog(PreferencesDialog):
    __gtype_name__ = "PreferencesArduinoScopeDialog"

    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the preferences dialog"""
        super(PreferencesArduinoScopeDialog, self).finish_initializing(builder)

        # populate the dialog from the preferences dictionary
        # using the methods from widget_methods
        self.widget_methods = widget_methods
        self.set_widgets_from_preferences() # pylint: disable=E1101

        # Code for other initialization actions should be added here.
        
    def _check_serial_connection(self):

        try:                  
            serial_port = preferences['serial_port']
            baudrate = preferences['baudrate']
            self.SerialConnection = Serial(serial_port,baudrate=baudrate , timeout=0) 
            self.SerialConnection.flushInput()
            self.SerialConnection.close()
        except StandardError, e:
            # delattr(self,"SerialConnection")
            logger.debug("Error while trying to test serial connectin: %s" % e)

    def on_serial_connection_state_canvas_exposed(self, widget=None, event=None):
        state = (0,1,0)
        if not hasattr(self, "SerialConnection"):
            state = (1,0,0)

        canvas = self.children()[0].children()[0].children()[0].children()[2].children()[0]

        canvas.window.clear()
        cr = canvas.window.cairo_create()
        cr.set_source_rgb(int(state[0]),int(state[1]),int(state[2]))
        cr.rectangle(0,0,50,50)
        cr.fill()
        

    def on_check_serial_connection_clicked(self, widget):
        self._check_serial_connection()
        self.on_serial_connection_state_canvas_exposed()



    def on_guess_samplerate_clicked(self, widget):
        progress_bar =  widget.parent.children()[0]
        fraction = 0.0
        progress_bar.set_fraction(fraction)
        progress_bar.set_text("")

        self._check_serial_connection()
        self.on_serial_connection_state_canvas_exposed
        if not hasattr(self, 'SerialConnection'):
            progress_bar.set_text("Could not connect.")
            return

        fraction += 0.2                
        # progress_bar.set_text("Scanning")
        progress_bar.set_fraction(fraction)

        self.SerialConnection.open()
        self.SerialConnection.flushInput()        
        while True:
            self.SerialConnection.flushInput()        
            self.SerialConnection.write('P\n')
            time.sleep(1.2)
            channels =  self.SerialConnection.readlines()
            logger.debug(channels)
            if len(channels) == 6:
                break
        
        fraction += 0.2                
        progress_bar.set_fraction(fraction)
        # progress_bar.set_text("calculating")
        

        hz = 0
        for a in channels:
            time.sleep(0.2)
            fraction += 0.1                
            progress_bar.set_fraction(fraction)
            o = a.split(":")           
            # channel = int(o[0][1:])
            val = int(o[1].strip('\r\n'))
            hz += val
          
        chz = float(hz / 6)
        hz = (500 / chz) * 1000
        progress_bar.set_text("%dHz (~%dms/500 samples)" % (int(hz),int(chz)))
