# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Steffen Kockel s.kockel@groovecubes.de
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import logging
logger = logging.getLogger('arduino_scope')

from serial import Serial
from arduino_scope_lib.preferences import preferences

def testme():
    pass

def check_serial_connection():
    connect = True
    try:                        
        SerialConnection = getSerialConnection()
        SerialConnection.close()
    except StandardError,e :
        logger.debug("Error while trying to test serial connectin: %s" % e)
        connect = False
    return connect
       
def getSerialConnection():
    serial_port = preferences['serial_port']
    baudrate = preferences['baudrate']
    SerialConnection = Serial(serial_port,baudrate=baudrate , timeout=0) 
    SerialConnection.flushInput()
    logger.debug("Established serial connection on %s with %s Baud/s" % (serial_port, baudrate))
    return SerialConnection

def drawPath(cr, xy, XY, width=1, color=(1,1,1)):
    """A shortcut to effeciently deal with the lot of pathes
       needed to get drawed.

       @param cr: graphical context
       @param xy: start, tuple of int, like (0,20)            
       @param XY: stop, tuple of int, like (0,20)
       @param width: int
       @param color: tuple of int, like (0,20,1)          
    """
    cr.move_to(xy[0],xy[1])
    cr.line_to(XY[0],XY[1])
    cr.close_path()
    cr.set_source_rgb(color[0],color[1],color[2])
    cr.set_line_width(width)
    cr.stroke()       

def drawText(cr, xy, text, fsize=13, fface="Sans", color=(0,0,0)):
    """Like drawPath, but for fonts.
    
    @param cr: graphical context
    @param xy: tuple of ints for xy 
    @param text: str 
    @param fsize: int
    @param fface: str
    @param color: tuple
    """
    cr.move_to(xy[0],xy[1])
    cr.set_source_rgb(color[0], color[1], color[2])
    cr.set_font_size(fsize)
    cr.select_font_face(fface)
    cr.show_text(text)            
    cr.stroke()

def getSamplerate(format="ms", ptype="float"):
    """ get the samplerate either in ms or Hz

        @param format: str  "ms" or "hz"
        @param format: str  "int" or "float"  
    """
    if format == "ms":
        samplerate = 500 / (int(preferences['samplerate']) / 1000)
        if ptype == "float":
            return float(samplerate)
        return samplerate
    elif format == "hz":
        samplerate = int(preferences['samplerate'])
        if ptype == "float":
            return float(samplerate)
        return samplerate
